{% load thumbnail %}{% spaceless %}
{% thumbnail "invalid_image.jpg" "30x30" crop="50% 50%" as im %}
    <img style="margin:{{ im|margin:"30x30" }}" width="{{ im.x }}" height="{{ im.y }}">
{% empty %}
    <p>empty</p>
{% endthumbnail %}
{% endspaceless %}
