{% load thumbnail %}{% spaceless %}

{% thumbnail anything "200x100" as im %}
    <img style="margin:{{ im|margin:"200x100" }}" width="{{ im.x }}" height="{{ im.y }}">
{% endthumbnail %}

{% endspaceless %}
