{% load thumbnail %}{% spaceless %}
{% thumbnail "http://dummyimage.com/100x100/" "20x20" crop="center" as im %}
    <img style="margin:{{ im|margin:"20x20" }}" width="{{ im.x }}" height="{{ im.y }}">
{% endthumbnail %}
{% endspaceless %}
